#include "guimodule.h"
#include "windows.h"
#include <stdio.h>
#include <assert.h>

GuiModule::GuiModule(seGuiCallback seaudioMaster, void *p_resvd1) : SEGUI_base(seaudioMaster, p_resvd1)
,m_inhibit_update(false)
{
}

GuiModule::~GuiModule(void)
{
	delete random_table;
}

void GuiModule::Initialise(bool loaded_from_file)
{
	SEGUI_base::Initialise(loaded_from_file);

	RegisterPatchParameter();

	if( !loaded_from_file )
	{
		AddGuiPlug( DT_BOOL, DR_IN, "Gate" );
		AddGuiPlug( DT_BOOL, DR_IN, "Mode" );
		AddGuiPlug( DT_FLOAT, DR_IN, "Seed" );
		AddGuiPlug( DT_FLOAT, DR_PARAMETER, "Low" );
		AddGuiPlug( DT_FLOAT, DR_PARAMETER, "High" );
		AddGuiPlug( DT_FLOAT, DR_OUT, "Value" );
		oldseed = 1;
		getPin( 2 )->setValueFloat(1.f);
		getPin( 4 )->setValueFloat(1.f);

	}

	oldgate = 0;
	
	if( loaded_from_file )
	{
	oldseed = (int) getPin( 2 )->getValueFloat();
	}

	numclicks = 1;

	random_table = new float [512*sizeof(float)];

	FillTable((int) getPin( 2 )->getValueFloat());
}

void GuiModule::RegisterPatchParameter()
{
}

void GuiModule::FillTable(int seedvalue)
{
	srand( seedvalue );

	for (int i=0; i<512 ; i++) 
	{
		random_table[i]= (float) rand()/RAND_MAX;
	}

}

void GuiModule::OnGuiPinValueChange(SeGuiPin *p_pin)
{
	switch ( p_pin->getIndex() ) {
        case    0    :                        // GUI Pin number 0

			newgate = getPin( 0 )->getValueInt();
			mode = getPin( 1 )->getValueInt();
			randnum = random_table[numclicks&511];
			
			if (mode == 1)
			{
				if (randnum >= 0.5f)
					randnum = getPin( 4 )->getValueFloat();
				else if (randnum < 0.5f)
					randnum = getPin( 3 )->getValueFloat();
			}

			if ((newgate > 0) && (newgate != oldgate))
			{
			getPin( 5 )->setValueFloat(randnum);
			numclicks++;
			}

			oldgate = newgate;
            break;

        case    1    :                        // GUI Pin number 1

			break;

		case    2    :                        // GUI Pin number 2

			newseed = (int) getPin( 2 )->getValueFloat();

			if ( newseed != oldseed )
			{
				FillTable( newseed );
				oldseed = newseed;
			}

            break;
        }
}

float GuiModule::StringToFloat(const SeSdkString &p_string)
{
	char * temp;	// convert string to float
	return (float) strtod( p_string.data(), &temp );
}


void GuiModule::OnModuleMsg(int p_length, void * p_data)
{/*
	assert( p_length == sizeof(values) );
	memcpy(values,p_data,p_length);

	// aknowledge
	int h = Handle();
	char *msg = "ack";
	int size = 3;
////////////// EXPERIMENTAL ////////////
SendStringToAudio( 4, &h );
SendStringToAudio( 4, &size );
SendStringToAudio( size, msg );
///////////////////////////////////////

	InvalidateControl();*/
}

void GuiModule::SendStringToAudio( int p_length, void *p_data )
{
	CallHost(seGuiHostSendStringToAudio, p_length, 0, p_data );
}

int GuiModule::Handle(void)
{
	return CallHost(seGuiHostGetHandle);
}




